@@ -2,8 +2,8 @@ use std::env;
 use std::io::Write;
 use std::time::Duration;
 
+use clap::Subcommand;
 use eyre::Result;
-use structopt::StructOpt;
 use tabwriter::TabWriter;
 
 use atuin_client::database::Database;
@@ -11,51 +11,42 @@ use atuin_client::history::History;
 use atuin_client::settings::Settings;
 use atuin_client::sync;
 
-#[derive(StructOpt)]
+#[derive(Subcommand)]
+#[clap(infer_subcommands = true)]
 pub enum Cmd {
-    #[structopt(
-        about="begins a new command in the history",
-        aliases=&["s", "st", "sta", "star"],
-    )]
+    /// Begins a new command in the history
     Start { command: Vec<String> },
 
-    #[structopt(
-        about="finishes a new command in the history (adds time, exit code)",
-        aliases=&["e", "en"],
-    )]
+    /// Finishes a new command in the history (adds time, exit code)
     End {
         id: String,
-        #[structopt(long, short)]
+        #[clap(long, short)]
         exit: i64,
     },
 
-    #[structopt(
-        about="list all items in history",
-        aliases=&["l", "li", "lis"],
-    )]
+    /// List all items in history
     List {
-        #[structopt(long, short)]
+        #[clap(long, short)]
         cwd: bool,
 
-        #[structopt(long, short)]
+        #[clap(long, short)]
         session: bool,
 
-        #[structopt(long, short)]
+        #[clap(long)]
         human: bool,
 
-        #[structopt(long, help = "Show only the text of the command")]
+        /// Show only the text of the command
+        #[clap(long)]
         cmd_only: bool,
     },
 
-    #[structopt(
-        about="get the last command ran",
-        aliases=&["la", "las"],
-    )]
+    /// Get the last command ran
     Last {
-        #[structopt(long, short)]
+        #[clap(long)]
         human: bool,
 
-        #[structopt(long, help = "Show only the text of the command")]
+        /// Show only the text of the command
+        #[clap(long)]
         cmd_only: bool,
     },
 }
@@ -114,7 +105,12 @@ impl Cmd {
                     return Ok(());
                 }
 
-                let cwd = env::current_dir()?.display().to_string();
+                // It's better for atuin to silently fail here and attempt to
+                // store whatever is ran, than to throw an error to the terminal
+                let cwd = match env::current_dir() {
+                    Ok(dir) => dir.display().to_string(),
+                    Err(_) => String::from(""),
+                };
 
                 let h = History::new(chrono::Utc::now(), command, cwd, -1, -1, None, None);
 
@@ -174,7 +170,7 @@ impl Cmd {
                 let history = match (session, cwd) {
                     (None, None) => db.list(None, false).await?,
                     (None, Some(cwd)) => {
-                        let query = format!("select * from history where cwd = {};", cwd);
+                        let query = format!("select * from history where cwd = '{}';", cwd);
                         db.query_history(&query).await?
                     }
                     (Some(session), None) => {
@@ -183,7 +179,7 @@ impl Cmd {
                     }
                     (Some(session), Some(cwd)) => {
                         let query = format!(
-                            "select * from history where cwd = {} and session = {};",
+                            "select * from history where cwd = '{}' and session = {};",
                             cwd, session
                         );
                         db.query_history(&query).await?
