@@ -1,11 +1,13 @@
-use structopt::StructOpt;
+use clap::Parser;
 
-#[derive(StructOpt)]
+#[derive(Parser)]
 pub enum Cmd {
-    #[structopt(about = "zsh setup")]
+    /// Zsh setup
     Zsh,
-    #[structopt(about = "bash setup")]
+    /// Bash setup
     Bash,
+    /// Fish setup
+    Fish,
 }
 
 fn init_zsh() {
@@ -18,11 +20,17 @@ fn init_bash() {
     println!("{}", full);
 }
 
+fn init_fish() {
+    let full = include_str!("../shell/atuin.fish");
+    println!("{}", full);
+}
+
 impl Cmd {
     pub fn run(&self) {
         match self {
             Self::Zsh => init_zsh(),
             Self::Bash => init_bash(),
+            Self::Fish => init_fish(),
         }
     }
 }
